/* This is "heat5.c", written to accompany Introduction to
   Scientific Programming by Joseph L. Zachary. */

/* This program is an implementation and test driver for the
   heat transfer problem described in Chapter 17 of the text. */


#include <stdio.h>

#define segs 6      /* The number of segments to simulate.  This is */
                    /* all that must be modified to change the */
                    /* size of the simulation. */



/* This function models heat transfer in a rod of uniform temperature
   that is insulated along its lengths and has a heat source applied
   to its ends.  The rod's thermal "diffusivity" in cm^2/sec and
   "length" in cm are given, as are the temperature of the "rod", the
   "left" heat source, and the "right" heat source in degrees Celsius.

   The function returns the approximate temperature of the center of
   the rod in degrees Celsius "duration" seconds after the heat sources
   are applied.

   The implementation is based on a finite-element model with six
   elements. */

double simulate (double diffusivity, double length,
                 double left, double right, double rod,
                 double duration)
{

  double delta;        /* Length in seconds of each simulated time step */

  double cur[segs+1],  /* Temperatures at the ends of each of the six */
                       /* segments, where cur[0] is the temperature of */
                       /* the left heat source and cur[6] is the */
                       /* temperature of the right heat source. */

         nxt[segs+1];  /* Used to temporarily hold the new temperatures */
                       /* of "cur" during the simulation. */

  int i;               /* Steps through "cur" and "nxt". */


  /* Compute the length of the simulated time step. */

  delta = (length*length/(segs*segs)) / (2*diffusivity);


  /* Initialize "cur" to reflect the temperatures at time zero. */

  cur[0] = left;
  for (i = 1; i < segs; i = i+1) {
    cur[i] = rod;
  }
  cur[segs] = right;


  /* Simulate the passage of time.  During each iteration, compute the
     new segment boundary temperatures in terms of the old segment
     boundary temperatures. */

  while (duration > 0) {

    for (i = 1; i < segs; i = i+1) {
      nxt[i] = (cur[i-1] + cur[i+1]) / 2;
    }

    for (i = 1; i < segs; i = i+1) {
      cur[i] = nxt[i];
    }

    duration = duration - delta;
  }


  /* Return the temperature of the center segment boundary. */

  return(cur[segs/2]);

}



/* Test driver for "simulate". */

void main (void)
{

  double C, length, left,   /* Constants specified by the user. */
         right, rod, time,
         temp;


  /* Read from the keyboard using "scanf". */

  printf("Enter thermal diffusivity (cm^2/sec): ");
  scanf("%lf", &C);

  printf("Enter length of rod (cm): ");
  scanf("%lf", &length);

  printf("Enter temperature at left end (degrees C): ");
  scanf("%lf", &left);

  printf("Enter temperature of rod (degrees C): ");
  scanf("%lf", &rod);

  printf("Enter temperature at right end (degrees C): ");
  scanf("%lf", &right);

  printf("Enter length of simulation (sec): ");
  scanf("%lf", &time);


  /* Calculate and display the result. */

  temp = simulate(C, length, left, right, rod, time);
  printf("Temperature at center after %g seconds is %g degrees C\n",
         time, temp);

}
